# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Dict, Any, List

from permguard.az.azreq.model import AZRequest, Principal
from permguard.az.azreq.builder_subject import SubjectBuilder
from permguard.az.azreq.builder_resource import ResourceBuilder
from permguard.az.azreq.builder_action import ActionBuilder
from permguard.az.azreq.builder_context import ContextBuilder
from permguard.az.azreq.builder_request_composed import AZRequestBuilder


# Costanti per i tipi di subject
USER_TYPE = "user"
ROLE_ACTOR_TYPE = "role-actor"
TWIN_ACTOR_TYPE = "twin-actor"


class AZAtomicRequestBuilder:
    """Builder for creating an AZAtomicRequest object."""

    def __init__(self, zone_id: int, ledger_id: str, subject_id: str, resource_kind: str, action_name: str):
        """Initialize the builder with mandatory fields."""
        self._az_request_builder = AZRequestBuilder(zone_id, ledger_id)
        self._az_subject_builder = SubjectBuilder(subject_id)
        self._az_resource_builder = ResourceBuilder(resource_kind)
        self._az_action_builder = ActionBuilder(action_name)
        self._az_context_builder = ContextBuilder()
        self._request_id: str = ""
        self._principal: Principal = None

    def with_entities_map(self, schema: str, entities: Dict[str, Any]) -> "AZAtomicRequestBuilder":
        """Set the entities map in the AZRequest."""
        self._az_request_builder.with_entities_map(schema, entities)
        return self

    def with_entities_items(self, schema: str, entities: List[Dict[str, Any]]) -> "AZAtomicRequestBuilder":
        """Set the entities items in the AZRequest."""
        self._az_request_builder.with_entities_items(schema, entities)
        return self

    def with_request_id(self, request_id: str) -> "AZAtomicRequestBuilder":
        """Set the request ID for the AZRequest."""
        self._request_id = request_id
        return self

    def with_principal(self, principal: Principal) -> "AZAtomicRequestBuilder":
        """Set the principal for the AZRequest."""
        self._principal = principal
        return self

    def with_subject_user_type(self) -> "AZAtomicRequestBuilder":
        """Set the subject type to USER."""
        self._az_subject_builder.with_type(USER_TYPE)
        return self

    def with_subject_role_actor_type(self) -> "AZAtomicRequestBuilder":
        """Set the subject type to ROLE-ACTOR."""
        self._az_subject_builder.with_type(ROLE_ACTOR_TYPE)
        return self

    def with_subject_twin_actor_type(self) -> "AZAtomicRequestBuilder":
        """Set the subject type to TWIN-ACTOR."""
        self._az_subject_builder.with_type(TWIN_ACTOR_TYPE)
        return self

    def with_subject_type(self, kind: str) -> "AZAtomicRequestBuilder":
        """Set the subject type."""
        self._az_subject_builder.with_type(kind)
        return self

    def with_subject_source(self, source: str) -> "AZAtomicRequestBuilder":
        """Set the source of the subject."""
        self._az_subject_builder.with_source(source)
        return self

    def with_subject_property(self, key: str, value: Any) -> "AZAtomicRequestBuilder":
        """Set a property of the subject."""
        self._az_subject_builder.with_property(key, value)
        return self

    def with_resource_id(self, id: str) -> "AZAtomicRequestBuilder":
        """Set the resource ID."""
        self._az_resource_builder.with_id(id)
        return self

    def with_resource_property(self, key: str, value: Any) -> "AZAtomicRequestBuilder":
        """Set a property of the resource."""
        self._az_resource_builder.with_property(key, value)
        return self

    def with_action_property(self, key: str, value: Any) -> "AZAtomicRequestBuilder":
        """Set a property of the action."""
        self._az_action_builder.with_property(key, value)
        return self

    def with_context_property(self, key: str, value: Any) -> "AZAtomicRequestBuilder":
        """Set a property of the context."""
        self._az_context_builder.with_property(key, value)
        return self

    def build(self) -> AZRequest:
        """Build and return the AZRequest object."""
        subject = self._az_subject_builder.build()
        resource = self._az_resource_builder.build()
        action = self._az_action_builder.build()
        context = self._az_context_builder.build()

        self._az_request_builder \
            .with_principal(self._principal) \
            .with_request_id(self._request_id) \
            .with_subject(subject) \
            .with_resource(resource) \
            .with_action(action) \
            .with_context(context)

        return self._az_request_builder.build()



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Optional, Tuple

from permguard.internal.az.azreq.grpc.v1.pdp_client import authorization_check
from permguard.az_config import AZConfig, AZEndpoint
from permguard.az.azreq.model import AZRequest, AZResponse


class AZClient:
    """Client to interact with the authorization server."""

    def __init__(self, *opts):
        """Initialize the authorization client with optional configuration options."""
        self._az_config = AZConfig()
        self._az_config.pdp_endpoint = AZEndpoint(endpoint="localhost", port=9094)

        # Apply configuration options
        for opt in opts:
            opt(self._az_config)

    def check(self, req: AZRequest) -> Tuple[bool, Optional[AZResponse]]:
        """Check the input authorization request with the authorization server."""
        target = f"{self._az_config.pdp_endpoint.endpoint}:{self._az_config.pdp_endpoint.port}"
       
        can_execute = authorization_check(target, req)
        decision = can_execute.decision if can_execute else False
        return decision, can_execute



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Any
from permguard.az.azreq.model import Subject


# Valore predefinito per Subject
SUBJECT_DEFAULT_KIND = "user"

# Tipi di Subject
USER_TYPE = "user"
ROLE_ACTOR_TYPE = "role-actor"
TWIN_ACTOR_TYPE = "twin-actor"


class SubjectBuilder:
    """Builder for creating a Subject object."""

    def __init__(self, id: str):
        """Initialize the builder with a default subject type."""
        self._subject = Subject(id=id, type=SUBJECT_DEFAULT_KIND, properties={})

    def with_user_type(self) -> "SubjectBuilder":
        """Set the subject type to USER."""
        self._subject.type = USER_TYPE
        return self

    def with_role_actor_type(self) -> "SubjectBuilder":
        """Set the subject type to ROLE-ACTOR."""
        self._subject.type = ROLE_ACTOR_TYPE
        return self

    def with_twin_actor_type(self) -> "SubjectBuilder":
        """Set the subject type to TWIN-ACTOR."""
        self._subject.type = TWIN_ACTOR_TYPE
        return self

    def with_type(self, sub_type: str) -> "SubjectBuilder":
        """Set the subject type."""
        self._subject.type = sub_type
        return self

    def with_source(self, source: str) -> "SubjectBuilder":
        """Set the source of the subject."""
        self._subject.source = source
        return self

    def with_property(self, key: str, value: Any) -> "SubjectBuilder":
        """Set a property of the subject."""
        self._subject.properties[key] = value
        return self

    def build(self) -> Subject:
        """Build and return the Subject object."""
        return self._subject.model_copy(deep=True)



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Any

from permguard.az.azreq.model import Action


class ActionBuilder:
    """Builder for creating an Action object."""

    def __init__(self, name: str):
        """Initialize the builder with an action name."""
        self._action = Action(name=name, properties={})

    def with_property(self, key: str, value: Any) -> "ActionBuilder":
        """Set a property for the action."""
        self._action.properties[key] = value
        return self

    def build(self) -> Action:
        """Build and return the Action object with a deep copy of properties."""
        return self._action.model_copy(deep=True)



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from permguard.az.azreq.model import Principal


# Valore predefinito per Principal
PRINCIPAL_DEFAULT_KIND = "user"


class PrincipalBuilder:
    """Builder for creating a Principal object."""

    def __init__(self, id: str):
        """Initialize the builder with a default principal type."""
        self._principal = Principal(id=id, type=PRINCIPAL_DEFAULT_KIND)

    def with_kind(self, kind: str) -> "PrincipalBuilder":
        """Set the kind/type of the principal."""
        self._principal.type = kind
        return self

    def with_source(self, source: str) -> "PrincipalBuilder":
        """Set the source of the principal."""
        self._principal.source = source
        return self

    def build(self) -> Principal:
        """Build and return the Principal object."""
        return self._principal.model_copy(deep=True)



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

import copy
from typing import Any, Dict


class ContextBuilder:
    """Builder for creating a context dictionary."""

    def __init__(self):
        """Initialize the builder with an empty context dictionary."""
        self._context: Dict[str, Any] = {}

    def with_property(self, key: str, value: Any) -> "ContextBuilder":
        """Set a property for the context."""
        self._context[key] = value
        return self

    def build(self) -> Dict[str, Any]:
        """Build and return the context dictionary."""
        return copy.deepcopy(self._context)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: pdp.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'pdp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tpdp.proto\x12\x13policydecisionpoint\x1a\x1cgoogle/protobuf/struct.proto\"\'\n\x0bPolicyStore\x12\x0c\n\x04Kind\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\"\x9d\x01\n\tPrincipal\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x13\n\x06Source\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x1a\n\rIdentityToken\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x41\x63\x63\x65ssToken\x18\x05 \x01(\tH\x02\x88\x01\x01\x42\t\n\x07_SourceB\x10\n\x0e_IdentityTokenB\x0e\n\x0c_AccessToken\"B\n\x08\x45ntities\x12\x0e\n\x06Schema\x18\x01 \x01(\t\x12&\n\x05Items\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\"\x84\x01\n\x07Subject\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x13\n\x06Source\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x30\n\nProperties\x18\x04 \x01(\x0b\x32\x17.google.protobuf.StructH\x01\x88\x01\x01\x42\t\n\x07_SourceB\r\n\x0b_Properties\"e\n\x08Resource\x12\x0c\n\x04Type\x18\x01 \x01(\t\x12\n\n\x02ID\x18\x02 \x01(\t\x12\x30\n\nProperties\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x42\r\n\x0b_Properties\"W\n\x06\x41\x63tion\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\x30\n\nProperties\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x42\r\n\x0b_Properties\"\xeb\x01\n\x19\x41uthorizationModelRequest\x12\x0e\n\x06ZoneID\x18\x01 \x01(\x03\x12\x35\n\x0bPolicyStore\x18\x02 \x01(\x0b\x32 .policydecisionpoint.PolicyStore\x12\x36\n\tPrincipal\x18\x03 \x01(\x0b\x32\x1e.policydecisionpoint.PrincipalH\x00\x88\x01\x01\x12\x34\n\x08\x45ntities\x18\x04 \x01(\x0b\x32\x1d.policydecisionpoint.EntitiesH\x01\x88\x01\x01\x42\x0c\n\n_PrincipalB\x0b\n\t_Entities\"\xb4\x02\n\x11\x45valuationRequest\x12\x16\n\tRequestID\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x32\n\x07Subject\x18\x02 \x01(\x0b\x32\x1c.policydecisionpoint.SubjectH\x01\x88\x01\x01\x12\x34\n\x08Resource\x18\x03 \x01(\x0b\x32\x1d.policydecisionpoint.ResourceH\x02\x88\x01\x01\x12\x30\n\x06\x41\x63tion\x18\x04 \x01(\x0b\x32\x1b.policydecisionpoint.ActionH\x03\x88\x01\x01\x12-\n\x07\x43ontext\x18\x05 \x01(\x0b\x32\x17.google.protobuf.StructH\x04\x88\x01\x01\x42\x0c\n\n_RequestIDB\n\n\x08_SubjectB\x0b\n\t_ResourceB\t\n\x07_ActionB\n\n\x08_Context\"\xc5\x03\n\x19\x41uthorizationCheckRequest\x12J\n\x12\x41uthorizationModel\x18\x01 \x01(\x0b\x32..policydecisionpoint.AuthorizationModelRequest\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x32\n\x07Subject\x18\x03 \x01(\x0b\x32\x1c.policydecisionpoint.SubjectH\x01\x88\x01\x01\x12\x34\n\x08Resource\x18\x04 \x01(\x0b\x32\x1d.policydecisionpoint.ResourceH\x02\x88\x01\x01\x12\x30\n\x06\x41\x63tion\x18\x05 \x01(\x0b\x32\x1b.policydecisionpoint.ActionH\x03\x88\x01\x01\x12-\n\x07\x43ontext\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x04\x88\x01\x01\x12;\n\x0b\x45valuations\x18\x07 \x03(\x0b\x32&.policydecisionpoint.EvaluationRequestB\x0c\n\n_RequestIDB\n\n\x08_SubjectB\x0b\n\t_ResourceB\t\n\x07_ActionB\n\n\x08_Context\"/\n\x0eReasonResponse\x12\x0c\n\x04\x43ode\x18\x01 \x01(\t\x12\x0f\n\x07Message\x18\x02 \x01(\t\"\x90\x01\n\x0f\x43ontextResponse\x12\n\n\x02ID\x18\x01 \x01(\t\x12\x38\n\x0bReasonAdmin\x18\x02 \x01(\x0b\x32#.policydecisionpoint.ReasonResponse\x12\x37\n\nReasonUser\x18\x03 \x01(\x0b\x32#.policydecisionpoint.ReasonResponse\"\x94\x01\n\x12\x45valuationResponse\x12\x10\n\x08\x44\x65\x63ision\x18\x01 \x01(\x08\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12:\n\x07\x43ontext\x18\x03 \x01(\x0b\x32$.policydecisionpoint.ContextResponseH\x01\x88\x01\x01\x42\x0c\n\n_RequestIDB\n\n\x08_Context\"\xda\x01\n\x1a\x41uthorizationCheckResponse\x12\x10\n\x08\x44\x65\x63ision\x18\x01 \x01(\x08\x12\x16\n\tRequestID\x18\x02 \x01(\tH\x00\x88\x01\x01\x12:\n\x07\x43ontext\x18\x03 \x01(\x0b\x32$.policydecisionpoint.ContextResponseH\x01\x88\x01\x01\x12<\n\x0b\x45valuations\x18\x04 \x03(\x0b\x32\'.policydecisionpoint.EvaluationResponseB\x0c\n\n_RequestIDB\n\n\x08_Context2\x87\x01\n\x0cV1PDPService\x12w\n\x12\x41uthorizationCheck\x12..policydecisionpoint.AuthorizationCheckRequest\x1a/.policydecisionpoint.AuthorizationCheckResponse\"\x00\x42:Z8github.com/permguard/permguard/internal/hosts/api/pdp/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pdp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z8github.com/permguard/permguard/internal/hosts/api/pdp/v1'
  _globals['_POLICYSTORE']._serialized_start=64
  _globals['_POLICYSTORE']._serialized_end=103
  _globals['_PRINCIPAL']._serialized_start=106
  _globals['_PRINCIPAL']._serialized_end=263
  _globals['_ENTITIES']._serialized_start=265
  _globals['_ENTITIES']._serialized_end=331
  _globals['_SUBJECT']._serialized_start=334
  _globals['_SUBJECT']._serialized_end=466
  _globals['_RESOURCE']._serialized_start=468
  _globals['_RESOURCE']._serialized_end=569
  _globals['_ACTION']._serialized_start=571
  _globals['_ACTION']._serialized_end=658
  _globals['_AUTHORIZATIONMODELREQUEST']._serialized_start=661
  _globals['_AUTHORIZATIONMODELREQUEST']._serialized_end=896
  _globals['_EVALUATIONREQUEST']._serialized_start=899
  _globals['_EVALUATIONREQUEST']._serialized_end=1207
  _globals['_AUTHORIZATIONCHECKREQUEST']._serialized_start=1210
  _globals['_AUTHORIZATIONCHECKREQUEST']._serialized_end=1663
  _globals['_REASONRESPONSE']._serialized_start=1665
  _globals['_REASONRESPONSE']._serialized_end=1712
  _globals['_CONTEXTRESPONSE']._serialized_start=1715
  _globals['_CONTEXTRESPONSE']._serialized_end=1859
  _globals['_EVALUATIONRESPONSE']._serialized_start=1862
  _globals['_EVALUATIONRESPONSE']._serialized_end=2010
  _globals['_AUTHORIZATIONCHECKRESPONSE']._serialized_start=2013
  _globals['_AUTHORIZATIONCHECKRESPONSE']._serialized_end=2231
  _globals['_V1PDPSERVICE']._serialized_start=2234
  _globals['_V1PDPSERVICE']._serialized_end=2369
# @@protoc_insertion_point(module_scope)



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

import grpc
from google.protobuf.struct_pb2 import Struct
from google.protobuf.json_format import MessageToJson

from permguard.az.azreq.model import AZRequest, AZResponse
from permguard.internal.az.azreq.grpc.v1 import pdp_pb2, pdp_pb2_grpc


def dict_to_struct(data: dict) -> Struct:
    """Convert a Python dictionary to a Protobuf Struct."""
    struct = Struct()
    struct.update(data)
    return struct


def map_az_request_to_grpc(az_request: AZRequest) -> pdp_pb2.AuthorizationCheckRequest:
    """Map AZRequest (Pydantic) to AuthorizationCheckRequest (gRPC Protobuf)."""
    
    if az_request is None:
        raise ValueError("Invalid AZRequest: input is None")


    # Authorization Model
    auth_model = None
    if az_request.authorization_model:
        policy_store = None
        if az_request.authorization_model.policy_store:
            policy_store = pdp_pb2.PolicyStore(
                Kind=az_request.authorization_model.policy_store.kind,
                ID=az_request.authorization_model.policy_store.id,
            )
            if policy_store.Kind is None or policy_store.Kind == '':
                policy_store.Kind = 'ledger'

        principal = None
        if az_request.authorization_model.principal:
            principal = pdp_pb2.Principal(
                Type=az_request.authorization_model.principal.type,
                ID=az_request.authorization_model.principal.id,
                Source=az_request.authorization_model.principal.source,
            )

        entities = None
        if az_request.authorization_model.entities:
            entities = pdp_pb2.Entities(
                Schema=az_request.authorization_model.entities.schema_name,
                Items=[dict_to_struct(item) for item in az_request.authorization_model.entities.items],
            )

        auth_model = pdp_pb2.AuthorizationModelRequest(
            ZoneID=az_request.authorization_model.zone_id,
            PolicyStore=policy_store,
            Principal=principal,
            Entities=entities,
        )

    # Subject
    subject = None
    if az_request.subject:
        subject = pdp_pb2.Subject(
            Type=az_request.subject.type,
            ID=az_request.subject.id,
            Source=az_request.subject.source,
            Properties=dict_to_struct(az_request.subject.properties) if az_request.subject.properties else Struct(),
        )

    # Resource
    resource = None
    if az_request.resource:
        resource = pdp_pb2.Resource(
            Type=az_request.resource.type,
            ID=az_request.resource.id,
            Properties=dict_to_struct(az_request.resource.properties) if az_request.resource.properties else Struct(),
        )

    # Action
    action = None
    if az_request.action:
        action = pdp_pb2.Action(
            Name=az_request.action.name,
            Properties=dict_to_struct(az_request.action.properties) if az_request.action.properties else Struct(),
        )

    # Context
    context = dict_to_struct(az_request.context) if az_request.context else Struct()

    # Evaluations
    evaluations = []
    if az_request.evaluations:
        for eval in az_request.evaluations:
            eval_subject = None
            if eval.subject:
                eval_subject = pdp_pb2.Subject(
                    Type=eval.subject.type,
                    ID=eval.subject.id,
                    Source=eval.subject.source,
                    Properties=dict_to_struct(eval.subject.properties) if eval.subject.properties else Struct(),
                )

            eval_resource = None
            if eval.resource:
                eval_resource = pdp_pb2.Resource(
                    Type=eval.resource.type,
                    ID=eval.resource.id,
                    Properties=dict_to_struct(eval.resource.properties) if eval.resource.properties else Struct(),
                )

            eval_action = None
            if eval.action:
                eval_action = pdp_pb2.Action(
                    Name=eval.action.name,
                    Properties=dict_to_struct(eval.action.properties) if eval.action.properties else Struct(),
                )

            eval_context = dict_to_struct(eval.context) if eval.context else Struct()

            evaluations.append(
                pdp_pb2.EvaluationRequest(
                    RequestID=eval.request_id,
                    Subject=eval_subject,
                    Resource=eval_resource,
                    Action=eval_action,
                    Context=eval_context,
                )
            )

    # Creazione della richiesta gRPC
    return pdp_pb2.AuthorizationCheckRequest(
        AuthorizationModel=auth_model,
        RequestID=az_request.request_id,
        Subject=subject,
        Resource=resource,
        Action=action,
        Context=context,
        Evaluations=evaluations,
    )


def authorization_check(endpoint: str, az_request: AZRequest) -> AZResponse:
    """Execute gRPC authorization check and return AZResponse."""
    if az_request is None:
        raise ValueError("PEP: Invalid request")


    with grpc.insecure_channel(endpoint) as channel:
        stub = pdp_pb2_grpc.V1PDPServiceStub(channel)

        grpc_request = map_az_request_to_grpc(az_request)
        print(MessageToJson(grpc_request))
        grpc_response = stub.AuthorizationCheck(grpc_request)

        reponse_json = MessageToJson(grpc_response)
        response = AZResponse.model_validate_json(reponse_json)
        return response



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any


class PolicyStore(BaseModel):
    kind: Optional[str] = None
    id: Optional[str] = None


class Entities(BaseModel):
    schema_name: Optional[str] = Field(default='', alias="schema")  
    items: List[Dict[str, Any]] = []


class Principal(BaseModel):
    type: Optional[str] = None
    id: Optional[str] = None
    source: Optional[str] = None


class Subject(BaseModel):
    type: Optional[str] = None
    id: Optional[str] = None
    source: Optional[str] = None
    properties: Dict[str, Any] = {}


class Resource(BaseModel):
    type: Optional[str] = None
    id: Optional[str] = None
    properties: Dict[str, Any] = {}


class Action(BaseModel):
    name: Optional[str] = None
    properties: Dict[str, Any] = {}


class Evaluation(BaseModel):
    request_id: Optional[str] = None
    subject: Optional[Subject] = None
    resource: Optional[Resource] = None
    action: Optional[Action] = None
    context: Dict[str, Any] = {}


class AZModel(BaseModel):
    zone_id: int
    principal: Optional[Principal] = None
    policy_store: Optional[PolicyStore] = None
    entities: Optional[Entities] = None


class AZRequest(BaseModel):
    authorization_model: Optional[AZModel] = None
    request_id: str = Field(default='')
    subject: Optional[Subject] = None
    resource: Optional[Resource] = None
    action: Optional[Action] = None
    context: Dict[str, Any] = {}
    evaluations: List[Evaluation] = []


class ReasonResponse(BaseModel):
    code: Optional[str] =  Field(default=None, alias="Code") 
    message: Optional[str] =  Field(default=None, alias="Message") 


class ContextResponse(BaseModel):
    id: Optional[str] = Field(None, alias="ID")
    reason_admin: Optional[ReasonResponse] = Field(default=None, alias="ReasonAdmin") 
    reason_user: Optional[ReasonResponse] = Field(default=None, alias="ReasonUser") 


class EvaluationResponse(BaseModel):
    request_id: str = Field(default='', alias="RequestID")
    decision: bool = Field(default=False, alias="Decision")
    context: Optional[ContextResponse] = Field(default=None, alias="Context") 


class AZResponse(BaseModel):
    request_id: str = Field(default='', alias="RequestID")
    decision: bool = Field(default=False, alias="Decision")
    context: Optional[ContextResponse] = Field(default=None, alias="Context") 
    evaluations: List[EvaluationResponse] = Field(default=[], alias="Evaluations") 



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Callable, Optional


class AZEndpoint:
    """Represents the authorization server endpoint."""
    
    def __init__(self, endpoint: str, port: int):
        self.endpoint = endpoint
        self.port = port


class AZConfig:
    """Configuration for the authorization client."""
    
    def __init__(self):
        self.pdp_endpoint: Optional[AZEndpoint] = None

    def apply(self, option: Callable[["AZConfig"], None]) -> "AZConfig":
        """Apply an AZOption function to modify the configuration."""
        option(self)
        return self


def with_endpoint(endpoint: str, port: int) -> Callable[[AZConfig], None]:
    """Set the gRPC endpoint for the authorization server."""
    return lambda c: setattr(c, "pdp_endpoint", AZEndpoint(endpoint, port))



# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Dict, Any, List
from permguard.az.azreq.model import (
    AZRequest, AZModel, PolicyStore, Entities, Principal, Subject, Resource, Action, Evaluation
)


# Costanti per i tipi di store
POLICY_STORE_KIND = "ledger"
CEDAR_ENTITY_KIND = "cedar"


class AZRequestBuilder:
    """Builder for creating an AZRequest object."""

    def __init__(self, zone_id: int, ledger_id: str):
        """Initialize the builder with a default AZModel."""
        self._az_request = AZRequest(
            authorization_model=AZModel(
                zone_id=zone_id,
                policy_store=PolicyStore(kind=POLICY_STORE_KIND, id=ledger_id),
                entities=Entities()
            ),
            evaluations=[]
        )

    def with_principal(self, principal: Principal) -> "AZRequestBuilder":
        """Set the principal for the AZRequest."""
        self._az_request.authorization_model.principal = principal
        return self

    def with_request_id(self, request_id: str) -> "AZRequestBuilder":
        """Set the request ID for the AZRequest."""
        self._az_request.request_id = request_id
        return self

    def with_subject(self, subject: Subject) -> "AZRequestBuilder":
        """Set the subject for the AZRequest."""
        self._az_request.subject = subject
        return self

    def with_resource(self, resource: Resource) -> "AZRequestBuilder":
        """Set the resource for the AZRequest."""
        self._az_request.resource = resource
        return self

    def with_action(self, action: Action) -> "AZRequestBuilder":
        """Set the action for the AZRequest."""
        self._az_request.action = action
        return self

    def with_context(self, context: Dict[str, Any]) -> "AZRequestBuilder":
        """Set the context for the AZRequest."""
        self._az_request.context = context
        return self

    def with_entities_map(self, schema: str, entities: Dict[str, Any]) -> "AZRequestBuilder":
        """Set the entities map in the AZRequest."""
        self._az_request.authorization_model.entities.schema_name = schema
        self._az_request.authorization_model.entities.items = [entities]
        return self

    def with_entities_items(self, schema: str, entities: List[Dict[str, Any]]) -> "AZRequestBuilder":
        """Set the entities items in the AZRequest."""
        self._az_request.authorization_model.entities.schema_name = schema
        self._az_request.authorization_model.entities.items = entities or []
        return self

    def with_evaluation(self, evaluation: Evaluation) -> "AZRequestBuilder":
        """Add an evaluation to the AZRequest."""
        self._az_request.evaluations.append(evaluation)
        return self

    def build(self) -> AZRequest:
        """Build and return the AZRequest object."""
        return self._az_request.model_copy(deep=True)




# Copyright 2025 Nitro Agility S.r.l.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

from typing import Dict, Any

from permguard.az.azreq.model import Evaluation, Subject, Resource, Action


class EvaluationBuilder:
    """Builder for creating an Evaluation object."""

    def __init__(self, subject: Subject, resource: Resource, action: Action):
        """Initialize the builder with required subject, resource, and action."""
        self._evaluation = Evaluation(
            subject=subject,
            resource=resource,
            action=action,
            context={}
        )

    def with_request_id(self, request_id: str) -> "EvaluationBuilder":
        """Set the request ID of the Evaluation."""
        self._evaluation.request_id = request_id
        return self

    def with_context(self, context: Dict[str, Any]) -> "EvaluationBuilder":
        """Set the context of the Evaluation."""
        self._evaluation.context = context
        return self

    def build(self) -> Evaluation:
        """Build and return the Evaluation object."""
        return self._evaluation.model_copy(deep=True)
